#!/usr/bin/env python

import re
from saml2 import server
from saml2 import saml
from saml2 import soap

# ----------------------------------------------------------------------------
# Attribute Authority Stuff
# ----------------------------------------------------------------------------

def _aa_reply(attribute_auth, aq, items, sp_entityid):
    consumer_url = attribute_auth.metadata.consumer_url(aq.issuer.text)
    in_response_to = aq.id
    name_id = aq.subject.name_id

    logging.info("name_id: %s" % name_id)
    return  attribute_auth.do_aa_response(in_response_to, consumer_url,
            sp_entityid,
            identity=items,
            name_id=name_id,
            issuer=attribute_auth.conf.entityid)

def aa(environ, start_response):
    logging.info("- Attribute Query -")
    #logging.info(environ)
    body = environ["wsgi.input"].read(int(environ["CONTENT_LENGTH"]))
    logging.info("Read SOAP message")
    aq_xml = soap.parse_soap_enveloped_saml_attribute_query(body)

    # Instanciate an AA instance a new one for every query
    # not the most efficient way
    attribute_auth = environ["vo.aa"]

    logging.info("Query before parsing: %s" % aq_xml)
    (subject, attribute, aq) = attribute_auth.parse_attribute_query(aq_xml,
                                                                    False)
    logging.info("Subject: %s" % subject)
    logging.info("Attribute: %s" % attribute)
    logging.info("Query: %s" % aq)
    sp_entityid = aq.issuer.text

    try:
        identity = environ["vo.userdb"][subject.text]
    except KeyError:
        return not_found(environ, start_response)

    logging.info("Identity: %s" % identity)

    # Construct the return message
    response = _aa_reply(attribute_auth, aq, identity, sp_entityid)
    logging.info("Response: %s" % response)
    xml = soap.make_soap_enveloped_saml_thingy(response)

    # return it to the requestor
    start_response('200 OK', [('Content-Type', 'application/soap+xml')])
    return [xml]

# ---------------------------------

#noinspection PyUnusedLocal
def not_found(environ, start_response):
    """Called if no URL matches."""
    start_response('404 NOT FOUND', [('Content-Type', 'text/plain')])
    return ['Not Found']

# ----------------------------------------------------------------------------

# map urls to functions
URLS = [
    (r'aa$', aa),
]

# ----------------------------------------------------------------------------

def application(environ, start_response):
    """
    The main WSGI application. Dispatch the current request to
    the functions from above and store the regular expression
    captures in the WSGI environment as  `myapp.url_args` so that
    the functions from above can access the url placeholders.

    If nothing matches call the `not_found` function.
    
    :param environ: The HTTP application environment
    :param start_response: The application to run when the handling of the 
        request is done
    :return: The response as a list of lines
    """

    path = environ.get('PATH_INFO', '').lstrip('/')
    environ["vo.aa"] = AA
    environ["vo.userdb"] = USER_DB

    logging.info("<application> PATH: %s" % path)

    for regex, callback in URLS:
        match = re.search(regex, path)
        if match is not None:
            try:
                environ['myapp.url_args'] = match.groups()[0]
            except IndexError:
                environ['myapp.url_args'] = path
            if logging: logging.info("callback: %s" % (callback,))
            return callback(environ, start_response)
    return not_found(environ, start_response)

# ----------------------------------------------------------------------------

if __name__ == '__main__':
    import sys
    from wsgiref.simple_server import make_server
    import logging

    import vo_user
    
    PORT = 8088
    
    logging.basicConfig(filename="./idp.log", level=logging.DEBUG)

    USER_DB = vo_user.USER
    AA = server.Server(sys.argv[1], log=logging, debug=1, stype="aa")
    SRV = make_server('localhost', PORT, application)
    print "AA listening on port: %s" % PORT
    SRV.serve_forever()