#!/usr/bin/env python

import time
from saml2.cache import Cache
from saml2.time_util import in_a_while, str_to_time

SESSION_INFO_PATTERN = {"ava":{}, "came from":"", "not_on_or_after":0,
                    "issuer":"", "session_id":-1}


cache = Cache()

not_on_or_after = str_to_time(in_a_while(days=1))
session_info = SESSION_INFO_PATTERN.copy()
session_info["ava"] = {"givenName":["Derek"]}
cache.set("1234", "abcd", session_info,
                not_on_or_after)
                
(ava, inactive) = cache.get_identity("1234")

not_on_or_after = str_to_time(in_a_while(days=1))
session_info = SESSION_INFO_PATTERN.copy()
session_info["ava"] = {"surName":["Jeter"]}
cache.set("1234", "bcde", session_info, not_on_or_after)
                
(ava, inactive) = cache.get_identity("1234")

session_info = cache.get("1234","bcde")
ava = session_info["ava"]
session_info = cache.get("1234","abcd")
ava = session_info["ava"]
print ava

cache.reset("1234", "bcde")
assert cache.active("1234", "bcde") == False
assert cache.active("1234", "abcd")

(ava, inactive) = cache.get_identity("1234")
assert inactive == ['bcde']
assert ava["givenName"] == ["Derek"]

assert cache.active("1234", "bcde") == False
assert cache.active("1234", "abcd")

assert cache.subjects() == ["1234"]

not_on_or_after = str_to_time(in_a_while(days=1))
session_info = SESSION_INFO_PATTERN.copy()
session_info["ava"] = {"givenName":["Ichiro"],
                        "surName":["Suzuki"]}
cache.set("9876", "abcd", session_info,
                not_on_or_after)

(ava, inactive) = cache.get_identity("9876")
assert inactive == []
assert ava["givenName"] == ["Ichiro"]
assert ava["surName"] == ["Suzuki"]

not_on_or_after = str_to_time(in_a_while(days=1))
session_info = SESSION_INFO_PATTERN.copy()
session_info["ava"] = {"givenName":["Ichiro"],
                        "surName":["Suzuki"]}
cache.set("9876", "bcde", session_info,
                not_on_or_after)

not_on_or_after = str_to_time(in_a_while(seconds=1))
session_info = SESSION_INFO_PATTERN.copy()
session_info["ava"] = {"givenName":["Alex"],
                        "surName":["Rodriguez"]}
cache.set("1000", "bcde", session_info,
                not_on_or_after)
                
time.sleep(2)
(ava, inactive) = cache.get_identity("1000")
assert inactive == ["bcde"]
assert ava == {}

      